/*
 * Copyright (c) 2009 Lukas Mejdrech
 * All rights reserved.
 *
 * Redistribution and use in source and binary forms, with or without
 * modification, are permitted provided that the following conditions
 * are met:
 *
 * - Redistributions of source code must retain the above copyright
 *   notice, this list of conditions and the following disclaimer.
 * - Redistributions in binary form must reproduce the above copyright
 *   notice, this list of conditions and the following disclaimer in the
 *   documentation and/or other materials provided with the distribution.
 * - The name of the author may not be used to endorse or promote products
 *   derived from this software without specific prior written permission.
 *
 * THIS SOFTWARE IS PROVIDED BY THE AUTHOR ``AS IS'' AND ANY EXPRESS OR
 * IMPLIED WARRANTIES, INCLUDING, BUT NOT LIMITED TO, THE IMPLIED WARRANTIES
 * OF MERCHANTABILITY AND FITNESS FOR A PARTICULAR PURPOSE ARE DISCLAIMED.
 * IN NO EVENT SHALL THE AUTHOR BE LIABLE FOR ANY DIRECT, INDIRECT,
 * INCIDENTAL, SPECIAL, EXEMPLARY, OR CONSEQUENTIAL DAMAGES (INCLUDING, BUT
 * NOT LIMITED TO, PROCUREMENT OF SUBSTITUTE GOODS OR SERVICES; LOSS OF USE,
 * DATA, OR PROFITS; OR BUSINESS INTERRUPTION) HOWEVER CAUSED AND ON ANY
 * THEORY OF LIABILITY, WHETHER IN CONTRACT, STRICT LIABILITY, OR TORT
 * (INCLUDING NEGLIGENCE OR OTHERWISE) ARISING IN ANY WAY OUT OF THE USE OF
 * THIS SOFTWARE, EVEN IF ADVISED OF THE POSSIBILITY OF SUCH DAMAGE.
 */

/** @addtogroup arp
 *  @{
 */

/** @file
 *  ARP standalone module implementation.
 *  Contains skeleton module functions mapping.
 *  The functions are used by the module skeleton as module specific entry points.
 *  @see module.c
 */

#include <async.h>
#include <stdio.h>

#include <ipc/ipc.h>
#include <ipc/services.h>

#include "../../err.h"
#include "../../modules.h"

#include "arp.h"
#include "arp_module.h"

/** ARP module name.
 */
#define NAME	"ARP protocol"

/** Prints the module name.
 *  @see NAME
 */
void	arp_print_name( void );

/** Starts the ARP module.
 *  Initializes the client connection serving function, initializes the module, registers the module service and starts the async manager, processing IPC messages in an infinite loop.
 *  @param client_connection The client connection processing function. The module skeleton propagates its own one. Input parameter.
 *  @returns EOK on successful module termination.
 *  @returns Other error codes as defined for the arp_initialize() function.
 *  @returns Other error codes as defined for the REGISTER_ME() macro function.
 */
int	arp_start_module( async_client_conn_t client_connection );

/** ARP module global data.
 */
extern arp_globals_t	arp_globals;

void arp_print_name( void ){
	printf( NAME );
}

int arp_start_module( async_client_conn_t client_connection ){
	ERROR_DECLARE;

	ipcarg_t	phonehash;

	async_set_client_connection( client_connection );
	arp_globals.networking_phone = connect_to_service( SERVICE_NETWORKING );
	ERROR_PROPAGATE( arp_initialize());
	ERROR_PROPAGATE( REGISTER_ME( SERVICE_ARP, & phonehash ));

	async_manager();

	return EOK;
}

/** @}
 */
